@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 import os
+import logging
 from collections import OrderedDict
 from urllib.parse import urlencode
 
@@ -14,6 +15,9 @@ from .utils import (
 )
 
 
+logger = logging.getLogger('geo-alchemy')
+
+
 class GeoRouter(object):
     base_list_url = 'https://www.ncbi.nlm.nih.gov/geo/browse'
     base_detail_url = 'https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi'
@@ -57,7 +61,7 @@ class GeoRouter(object):
     def platform_list(self, zsort='date', display=20, page=1):
         return self._list('platforms', zsort, display, page)
 
-    def series_detail(self, accession, targ='self', form='xml', view='quick'):
+    def series_detail(self, accession, targ='all', form='xml', view='quick'):
         self._check_series_accession(accession)
         return self._detail(accession, targ, form, view)
 
@@ -227,16 +231,16 @@ class Column(object):
 
 class Platform(object):
     def __init__(
-            self, title, accession, technology,
-            distribution, organisms, manufacturer,
-            manufacturer_protocol, description,
-            columns, internal_data, release_date,
-            last_update_date, submission_date
+        self, accession, title=None, technology=None,
+        distribution=None, organisms=None, manufacturer=None,
+        manufacturer_protocol=None, description=None,
+        columns=None, internal_data=None, release_date=None,
+        last_update_date=None, submission_date=None
     ):
         """
         Args:
-            title: eg, [HG-U133_Plus_2] Affymetrix Human Genome U133 Plus 2.0 Array
             accession: eg, GPL570
+            title: eg, [HG-U133_Plus_2] Affymetrix Human Genome U133 Plus 2.0 Array
             technology: eg, in situ oligonucleotide
             distribution: eg, commercial
             organisms: correspond organisms, list of Organism objects
@@ -249,15 +253,15 @@ class Platform(object):
             last_update_date: last update date
             submission_date: submission date
         """
-        self.title = title
         self.accession = accession
+        self.title = title
         self.technology = technology
         self.distribution = distribution
-        self.organisms = organisms
+        self.organisms = organisms if organisms else []
         self.manufacturer = manufacturer
         self.manufacturer_protocol = manufacturer_protocol
         self.description = description
-        self.columns = columns
+        self.columns = columns if columns else []
         self.internal_data = internal_data
         self.release_date = release_date
         self.last_update_date = last_update_date
@@ -269,7 +273,7 @@ class Platform(object):
     def __eq__(self, other):
         if not isinstance(other, Platform):
             raise NotImplemented
-        return (self.title == other.title) and (self.accession == other.accession) and (
+        return (self.accession == other.accession) and (self.title == other.title) and (
                 self.technology == other.technology) and (self.distribution == other.distribution) and (
                        self.organisms == other.organisms) and (self.manufacturer == other.manufacturer) and (
                        self.manufacturer_protocol == other.manufacturer_protocol) and (
@@ -289,11 +293,41 @@ class Platform(object):
             'description': self.description,
             'columns': [column.to_dict() for column in self.columns],
             'internal_data': self.internal_data,
-            'release_date': self.release_date.strftime('%Y-%m-%d'),
-            'last_update_date': self.last_update_date.strftime('%Y-%m-%d'),
-            'submission_date': self.submission_date.strftime('%Y-%m-%d'),
+            'release_date': self.release_date.strftime(
+                '%Y-%m-%d'
+            ) if self.release_date else None,
+            'last_update_date': self.last_update_date.strftime(
+                '%Y-%m-%d'
+            ) if self.last_update_date else None,
+            'submission_date': self.submission_date.strftime(
+                '%Y-%m-%d'
+            ) if self.submission_date else None,
         }
 
+    def get_probe_gene_mapping(self, gene_col):
+        if not (self.columns and self.internal_data):
+            raise GeoAlchemyError(f'malformed platform {self.accession}')
+        if len(self.columns) != len(self.internal_data[0]):
+            raise GeoAlchemyError(f'malformed platform {self.accession}')
+        col_num = len(self.columns)
+        if (gene_col+1) >= col_num:
+            raise GeoAlchemyError(f'gene column {gene_col+1}, but only {col_num} totally.')
+        mapping = OrderedDict()
+        for i, row in enumerate(self.internal_data):
+            row_col_num = len(row)
+            if row_col_num != col_num:
+                logger.warning(
+                    f'malformed platform annotation row {i}, only {row_col_num} columns, '
+                    f'should have {col_num} columns normally.'
+                )
+                continue
+            mapping[row[0]] = row[gene_col]
+        return mapping
+
+    @property
+    def is_malformed(self):
+        return self.title is None
+
 
 class Characteristic(object):
     def __init__(self, tag, value):
@@ -317,10 +351,10 @@ class Characteristic(object):
 
 class Channel(object):
     def __init__(
-            self, position, source, organisms,
-            characteristics, treatment_protocol,
-            growth_protocol, molecule, extract_protocol,
-            label, label_protocol
+        self, position, source, organisms,
+        characteristics, treatment_protocol,
+        growth_protocol, molecule, extract_protocol,
+        label, label_protocol
     ):
         """
         Args:
@@ -380,24 +414,24 @@ class Channel(object):
 class Sample(object):
 
     def __init__(
-            self, title, accession, type,
-            channel_count, channels,
-            hybridization_protocol, scan_protocol,
-            description, data_processing, supplementary_data,
-            columns, internal_data, release_date,
-            last_update_date, submission_date, platform
+        self, accession, title=None, type=None,
+        channel_count=None, channels=None,
+        hybridization_protocol=None, scan_protocol=None,
+        description=None, data_processing=None, supplementary_data=None,
+        columns=None, internal_data=None, release_date=None,
+        last_update_date=None, submission_date=None, platform=None
     ):
-        self.title = title
         self.accession = accession
+        self.title = title
         self.type = type
         self.channel_count = channel_count
-        self.channels = channels
+        self.channels = channels if channels else []
         self.hybridization_protocol = hybridization_protocol
         self.scan_protocol = scan_protocol
         self.description = description
         self.data_processing = data_processing
-        self.supplementary_data = supplementary_data
-        self.columns = columns
+        self.supplementary_data = supplementary_data if supplementary_data else []
+        self.columns = columns if columns else []
         self.internal_data = internal_data
         self.release_date = release_date
         self.last_update_date = last_update_date
@@ -410,7 +444,7 @@ class Sample(object):
     def __eq__(self, other):
         if not isinstance(other, Sample):
             raise NotImplemented
-        return (self.title == other.title) and (self.accession == other.accession) and (
+        return (self.accession == other.accession) and (self.title == other.title) and (
                 self.type == other.type) and (self.channel_count == other.channel_count) and (
                        self.channels == other.channels) and (self.hybridization_protocol == other.hybridization_protocol) and (
                        self.scan_protocol == other.scan_protocol) and (self.description == other.description) and (
@@ -436,10 +470,16 @@ class Sample(object):
             ],
             'columns': [column.to_dict() for column in self.columns],
             'internal_data': self.internal_data,
-            'release_date': self.release_date.strftime('%Y-%m-%d'),
-            'last_update_date': self.last_update_date.strftime('%Y-%m-%d'),
-            'submission_date': self.submission_date.strftime('%Y-%m-%d'),
-            'platform': self.platform.to_dict()
+            'release_date': self.release_date.strftime(
+                '%Y-%m-%d'
+            ) if self.release_date else None,
+            'last_update_date': self.last_update_date.strftime(
+                '%Y-%m-%d'
+            ) if self.last_update_date else None,
+            'submission_date': self.submission_date.strftime(
+                '%Y-%m-%d'
+            ) if self.submission_date else None,
+            'platform': self.platform.to_dict() if self.platform else None
         }
 
     @property
@@ -459,11 +499,13 @@ class Sample(object):
     @property
     def clinical(self):
         data = {
-            'title': self.title,
             'accession': self.accession,
+            'title': self.title,
             'platform': self.platform.accession if self.platform else None,
         }
-        if self.channel_count == 1:
+        if self.channels is None:
+            return data
+        if len(self.channels) == 1:
             data['source'] = self.channels[0].source
             scinames = OrderedDict()
             for organism in self.channels[0].organisms:
@@ -533,24 +575,28 @@ class Sample(object):
             data['molecule'] = molecule
         return data
 
+    @property
+    def is_malformed(self):
+        return self.title is None
+
 
 class Series(object):
     def __init__(
-            self, title, accession, pmids, summary,
-            overall_design, experiment_types, supplementary_data,
-            release_date, last_update_date, submission_date, samples
+        self, accession, title=None, pmids=None, summary=None,
+        overall_design=None, experiment_types=None, supplementary_data=None,
+        release_date=None, last_update_date=None, submission_date=None, samples=None
     ):
-        self.title = title
         self.accession = accession
-        self.pmids = pmids
+        self.title = title
+        self.pmids = pmids if pmids else []
         self.summary = summary
         self.overall_design = overall_design
-        self.experiment_types = experiment_types
-        self.supplementary_data = supplementary_data
+        self.experiment_types = experiment_types if experiment_types else []
+        self.supplementary_data = supplementary_data if supplementary_data else []
         self.release_date = release_date
         self.last_update_date = last_update_date
         self.submission_date = submission_date
-        self.samples = samples
+        self.samples = samples if samples else []
 
     def __repr__(self):
         return f'Series<{self.accession}>'
@@ -560,10 +606,10 @@ class Series(object):
             raise NotImplemented
         return (self.title == other.title) and (self.accession == other.accession) and (
                 self.pmids == other.pmids) and (self.summary == other.summary) and (
-                       self.overall_design == other.overall_design) and (self.experiment_types == other.experiment_types) and (
-                       self.supplementary_data == other.supplementary_data) and (self.release_date == other.release_date) and (
-                       self.last_update_date == other.last_update_date) and (self.submission_date == other.submission_date) and (
-                       self.samples == other.samples)
+                self.overall_design == other.overall_design) and (self.experiment_types == other.experiment_types) and (
+                self.supplementary_data == other.supplementary_data) and (self.release_date == other.release_date) and (
+                self.last_update_date == other.last_update_date) and (self.submission_date == other.submission_date) and (
+                self.samples == other.samples)
 
     def to_dict(self):
         return {
@@ -579,22 +625,12 @@ class Series(object):
                 supplementary_data_item.to_dict(
                 ) for supplementary_data_item in self.supplementary_data
             ],
-            'release_date': self.release_date.strftime('%Y-%m-%d'),
-            'last_update_date': self.last_update_date.strftime('%Y-%m-%d'),
-            'submission_date': self.submission_date.strftime('%Y-%m-%d'),
+            'release_date': self.release_date.strftime('%Y-%m-%d') if self.release_date else None,
+            'last_update_date': self.last_update_date.strftime('%Y-%m-%d') if self.last_update_date else None,
+            'submission_date': self.submission_date.strftime('%Y-%m-%d') if self.submission_date else None,
             'samples': [sample.to_dict() for sample in self.samples],
         }
 
-    def set_samples(self, samples):
-        self.samples = samples
-
-    def add_sample(self, sample):
-        self.samples.append(sample)
-
-    @property
-    def sample_count(self):
-        return len(self.samples)
-
     @property
     def platforms(self):
         cache = set()
@@ -608,6 +644,16 @@ class Series(object):
             cache.add(sample.platform.accession)
         return platforms
 
+    def set_samples(self, samples):
+        self.samples = samples
+
+    def add_sample(self, sample):
+        self.samples.append(sample)
+
+    @property
+    def sample_count(self):
+        return len(self.samples)
+
     @property
     def organisms(self):
         cache = set()
@@ -668,3 +714,11 @@ class Series(object):
         else:
             downloader = HttpDownloader(DEFAULT_RETRIES)
             return downloader.dl(url, outfile=file)
+
+    @property
+    def clinical(self):
+        return [samples.clinical for samples in self.samples]
+
+    @property
+    def is_malformed(self):
+        return self.title is None
